@@ -6,13 +6,11 @@ toolchain="arm-bcm2708hardfp-linux-gnueabi"
 compiler="gcc-4.7.1-release"
 cpu="arm1176" # armv6
 
-third_party="libuv,libunwind,libatomic_ops,openfec,alsa,pulseaudio:5.0,speexdsp,sox,openssl,cpputest"
-
 scons -Q \
     --enable-werror \
     --enable-tests \
     --enable-examples \
-    --build-3rdparty=${third_party} \
+    --build-3rdparty=all,pulseaudio:5.0 \
     --host=${toolchain}
 
 find bin/${toolchain} -name 'roc-test-*' | \
