@@ -9,9 +9,11 @@ scons -Q --enable-werror --build-3rdparty=all \
       --disable-c11 \
       --disable-libunwind \
       --disable-openfec \
-      --disable-speex \
+      --disable-openssl \
+      --disable-speexdsp \
       --disable-sox \
-      --disable-pulseaudio
+      --disable-pulseaudio \
+      --disable-sndfile
 
 # optional dependencies: none, optional targets: all
 scons -Q --enable-werror --build-3rdparty=all \
@@ -21,9 +23,11 @@ scons -Q --enable-werror --build-3rdparty=all \
       --enable-examples \
       --disable-libunwind \
       --disable-openfec \
-      --disable-speex \
+      --disable-openssl \
+      --disable-speexdsp \
       --disable-sox \
       --disable-pulseaudio \
+      --disable-sndfile \
       test
 
 # optional dependencies: all, optional targets: all
