@@ -2,14 +2,15 @@
 
 set -euxo pipefail
 
-for comp in gcc-11 gcc-12 clang-11 clang-14
+for comp in gcc-4.8 clang-3.7
 do
     scons -Q \
           --enable-werror \
           --enable-tests \
           --enable-benchmarks \
           --enable-examples \
-          --build-3rdparty=openfec \
+          --enable-doxygen \
+          --build-3rdparty=libatomic_ops,openfec,openssl,cpputest,google-benchmark \
           --compiler=${comp} \
           test
 done
