@@ -8,8 +8,7 @@
 
 #include "roc_audio/sample.h"
 
-const roc::audio::PcmFormat roc::audio::SampleFormat(roc::audio::PcmCode_Float32,
-                                                     roc::audio::PcmEndian_Native);
+const roc::audio::PcmFormat roc::audio::Sample_RawFormat = roc::audio::PcmFormat_Float32;
 
-const roc::audio::sample_t roc::audio::SampleMin = -1;
-const roc::audio::sample_t roc::audio::SampleMax = 1;
+const roc::audio::sample_t roc::audio::Sample_Min = -1;
+const roc::audio::sample_t roc::audio::Sample_Max = 1;
