@@ -12,23 +12,23 @@
 #ifndef ROC_AUDIO_SAMPLE_H_
 #define ROC_AUDIO_SAMPLE_H_
 
-#include "roc_audio/pcm_format.h"
+#include "roc_audio/sample_format.h"
 #include "roc_core/stddefs.h"
 
 namespace roc {
 namespace audio {
 
-//! Audio sample.
+//! Raw audio sample.
 typedef float sample_t;
 
-//! Sample format description.
-extern const PcmFormat SampleFormat;
+//! Format description for raw audio samples.
+extern const PcmFormat Sample_RawFormat;
 
-//! Minimum possible value of a sample.
-extern const sample_t SampleMin;
+//! Minimum possible value of a raw sample.
+extern const sample_t Sample_Min;
 
-//! Maximum possible value of a sample.
-extern const sample_t SampleMax;
+//! Maximum possible value of a raw sample.
+extern const sample_t Sample_Max;
 
 } // namespace audio
 } // namespace roc
