@@ -15,8 +15,8 @@ namespace roc {
 namespace core {
 
 void panic(const char* module, const char* file, int line, const char* format, ...) {
-    Console::instance().println(Color_None, "%s", "");
-    Console::instance().println(Color_None, "%s:%d: error: roc_panic()", file, line);
+    console_println("%s", "");
+    console_println("%s:%d: error: roc_panic()", file, line);
 
     char message[256] = {};
     size_t message_sz = sizeof(message) - 1;
