@@ -14,14 +14,20 @@
 namespace roc {
 namespace core {
 
-Console::Console() {
-}
-
-bool Console::colors_supported() {
+bool console_supports_colors() {
     return false;
 }
 
-void Console::println(Color, const char* format, ...) {
+void console_println(const char* format, ...) {
+    va_list args;
+    va_start(args, format);
+
+    __android_log_vprint(ANDROID_LOG_DEBUG, "roc", format, args);
+
+    va_end(args);
+}
+
+void console_println(Color color, const char* format, ...) {
     va_list args;
     va_start(args, format);
 
