@@ -42,6 +42,8 @@ public:
                                  const char* path,
                                  const Config& config,
                                  core::IArena& arena);
+    //! Returns name of backend.
+    virtual const char* name() const;
 
 private:
     bool first_created_;
