@@ -11,8 +11,8 @@
 #include "roc_address/interface.h"
 #include "roc_audio/channel_defs.h"
 #include "roc_audio/freq_estimator.h"
-#include "roc_audio/resampler_backend.h"
-#include "roc_audio/resampler_profile.h"
+#include "roc_audio/pcm_format.h"
+#include "roc_audio/resampler_config.h"
 #include "roc_core/attributes.h"
 #include "roc_core/log.h"
 
@@ -54,9 +54,9 @@ bool context_config_from_user(node::ContextConfig& out, const roc_context_config
 
 ROC_ATTR_NO_SANITIZE_UB
 bool sender_config_from_user(node::Context& context,
-                             pipeline::SenderConfig& out,
+                             pipeline::SenderSinkConfig& out,
                              const roc_sender_config& in) {
-    if (!sample_spec_from_user(out.input_sample_spec, in.frame_encoding)) {
+    if (!sample_spec_from_user(out.input_sample_spec, in.frame_encoding, false)) {
         roc_log(LogError, "bad configuration: invalid roc_sender_config.frame_encoding");
         return false;
     }
@@ -68,8 +68,9 @@ bool sender_config_from_user(node::Context& context,
                     " should be zero or valid encoding id");
             return false;
         }
-        const rtp::Format* format = context.format_map().find_by_pt(out.payload_type);
-        if (!format) {
+        const rtp::Encoding* encoding =
+            context.encoding_map().find_by_pt(out.payload_type);
+        if (!encoding) {
             roc_log(LogError,
                     "bad configuration: invalid roc_sender_config.packet_encoding:"
                     " no built-in or registered encoding found with id %u",
@@ -77,23 +78,32 @@ bool sender_config_from_user(node::Context& context,
             return false;
         }
     } else {
-        const rtp::Format* format =
-            context.format_map().find_by_spec(out.input_sample_spec);
-        if (!format) {
+        const rtp::Encoding* encoding =
+            context.encoding_map().find_by_spec(out.input_sample_spec);
+        if (!encoding) {
             roc_log(LogError,
                     "bad configuration:"
                     " failed to select packet_encoding matching frame_encoding,"
                     " set roc_sender_config.packet_encoding manually");
             return false;
         }
-        out.payload_type = format->payload_type;
+        out.payload_type = encoding->payload_type;
     }
 
     if (in.packet_length != 0) {
         out.packet_length = (core::nanoseconds_t)in.packet_length;
     }
 
+    if (in.target_latency != 0) {
+        out.latency.target_latency = (core::nanoseconds_t)in.target_latency;
+    }
+
+    if (in.latency_tolerance != 0) {
+        out.latency.latency_tolerance = (core::nanoseconds_t)in.latency_tolerance;
+    }
+
     out.enable_timing = false;
+    out.enable_auto_duration = true;
     out.enable_auto_cts = true;
 
     out.enable_interleaving = in.packet_interleaving;
@@ -117,14 +127,30 @@ bool sender_config_from_user(node::Context& context,
         return false;
     }
 
-    if (!resampler_backend_from_user(out.resampler_backend, in.resampler_backend)) {
+    if (!latency_tuner_backend_from_user(out.latency.tuner_backend,
+                                         in.latency_tuner_backend)) {
+        roc_log(LogError,
+                "bad configuration: invalid roc_sender_config.latency_tuner_backend:"
+                " should be valid enum value");
+        return false;
+    }
+
+    if (!latency_tuner_profile_from_user(out.latency.tuner_profile,
+                                         in.latency_tuner_profile)) {
+        roc_log(LogError,
+                "bad configuration: invalid roc_sender_config.latency_tuner_profile:"
+                " should be valid enum value");
+        return false;
+    }
+
+    if (!resampler_backend_from_user(out.resampler.backend, in.resampler_backend)) {
         roc_log(LogError,
                 "bad configuration: invalid roc_sender_config.resampler_backend:"
                 " should be valid enum value");
         return false;
     }
 
-    if (!resampler_profile_from_user(out.resampler_profile, in.resampler_profile)) {
+    if (!resampler_profile_from_user(out.resampler.profile, in.resampler_profile)) {
         roc_log(LogError,
                 "bad configuration: invalid roc_sender_config.resampler_profile:"
                 " should be valid enum value");
@@ -136,42 +162,31 @@ bool sender_config_from_user(node::Context& context,
 
 ROC_ATTR_NO_SANITIZE_UB
 bool receiver_config_from_user(node::Context&,
-                               pipeline::ReceiverConfig& out,
+                               pipeline::ReceiverSourceConfig& out,
                                const roc_receiver_config& in) {
     if (in.target_latency != 0) {
-        out.default_session.target_latency = (core::nanoseconds_t)in.target_latency;
+        out.session_defaults.latency.target_latency =
+            (core::nanoseconds_t)in.target_latency;
     }
 
     if (in.latency_tolerance != 0) {
-        out.default_session.latency_monitor.latency_tolerance =
+        out.session_defaults.latency.latency_tolerance =
             (core::nanoseconds_t)in.latency_tolerance;
-    } else {
-        out.default_session.latency_monitor.deduce_latency_tolerance(
-            out.default_session.target_latency);
     }
 
-    if (in.no_playback_timeout < 0) {
-        out.default_session.watchdog.no_playback_timeout = 0;
-    } else if (in.no_playback_timeout > 0) {
-        out.default_session.watchdog.no_playback_timeout = in.no_playback_timeout;
-    } else {
-        out.default_session.watchdog.deduce_no_playback_timeout(
-            out.default_session.target_latency);
+    if (in.no_playback_timeout != 0) {
+        out.session_defaults.watchdog.no_playback_timeout = in.no_playback_timeout;
     }
 
-    if (in.choppy_playback_timeout < 0) {
-        out.default_session.watchdog.choppy_playback_timeout = 0;
-    } else if (in.choppy_playback_timeout > 0) {
-        out.default_session.watchdog.choppy_playback_timeout = in.choppy_playback_timeout;
-
-        out.default_session.watchdog.deduce_choppy_playback_window(
-            out.default_session.watchdog.choppy_playback_timeout);
+    if (in.choppy_playback_timeout != 0) {
+        out.session_defaults.watchdog.choppy_playback_timeout =
+            in.choppy_playback_timeout;
     }
 
     out.common.enable_timing = false;
     out.common.enable_auto_reclock = true;
 
-    if (!sample_spec_from_user(out.common.output_sample_spec, in.frame_encoding)) {
+    if (!sample_spec_from_user(out.common.output_sample_spec, in.frame_encoding, false)) {
         roc_log(LogError,
                 "bad configuration: invalid roc_receiver_config.frame_encoding");
         return false;
@@ -184,42 +199,31 @@ bool receiver_config_from_user(node::Context&,
         return false;
     }
 
-    if (!clock_sync_backend_from_user(out.default_session.latency_monitor.fe_enable,
-                                      in.clock_sync_backend)) {
+    if (!latency_tuner_backend_from_user(out.session_defaults.latency.tuner_backend,
+                                         in.latency_tuner_backend)) {
         roc_log(LogError,
-                "bad configuration: invalid roc_receiver_config.clock_sync_backend:"
+                "bad configuration: invalid roc_receiver_config.latency_tuner_backend:"
                 " should be valid enum value");
         return false;
     }
 
-    if (in.clock_sync_profile != ROC_CLOCK_SYNC_PROFILE_DEFAULT) {
-        if (!clock_sync_profile_from_user(out.default_session.latency_monitor.fe_profile,
-                                          in.clock_sync_profile)) {
-            roc_log(LogError,
-                    "bad configuration: invalid roc_receiver_config.clock_sync_profile:"
-                    " should be valid enum value");
-            return false;
-        }
-    } else {
-        if (out.default_session.latency_monitor.fe_enable) {
-            out.default_session.latency_monitor.deduce_fe_profile(
-                out.default_session.target_latency);
-        }
+    if (!latency_tuner_profile_from_user(out.session_defaults.latency.tuner_profile,
+                                         in.latency_tuner_profile)) {
+        roc_log(LogError,
+                "bad configuration: invalid roc_receiver_config.latency_tuner_profile:"
+                " should be valid enum value");
+        return false;
     }
 
-    if (in.resampler_backend != ROC_RESAMPLER_BACKEND_DEFAULT) {
-        if (!resampler_backend_from_user(out.default_session.resampler_backend,
-                                         in.resampler_backend)) {
-            roc_log(LogError,
-                    "bad configuration: invalid roc_receiver_config.resampler_backend:"
-                    " should be valid enum value");
-            return false;
-        }
-    } else {
-        out.default_session.deduce_resampler_backend();
+    if (!resampler_backend_from_user(out.session_defaults.resampler.backend,
+                                     in.resampler_backend)) {
+        roc_log(LogError,
+                "bad configuration: invalid roc_receiver_config.resampler_backend:"
+                " should be valid enum value");
+        return false;
     }
 
-    if (!resampler_profile_from_user(out.default_session.resampler_profile,
+    if (!resampler_profile_from_user(out.session_defaults.resampler.profile,
                                      in.resampler_profile)) {
         roc_log(LogError,
                 "bad configuration: invalid roc_receiver_config.resampler_profile:"
@@ -230,33 +234,8 @@ bool receiver_config_from_user(node::Context&,
     return true;
 }
 
-ROC_ATTR_NO_SANITIZE_UB bool
-sender_interface_config_from_user(netio::UdpSenderConfig& out,
-                                  const roc_interface_config& in) {
-    if (in.outgoing_address[0] != '\0') {
-        if (!out.bind_address.set_host_port_auto(in.outgoing_address, 0)) {
-            roc_log(LogError,
-                    "bad configuration: invalid roc_interface_config.outgoing_address:"
-                    " should be either empty or valid IPv4/IPv6 address");
-            return false;
-        }
-    }
-
-    if (in.multicast_group[0] != '\0') {
-        roc_log(LogError,
-                "bad configuration: invalid roc_interface_config.multicast_group:"
-                " should be empty for sender");
-        return false;
-    }
-
-    out.reuseaddr = (in.reuse_address != 0);
-
-    return true;
-}
-
-ROC_ATTR_NO_SANITIZE_UB bool
-receiver_interface_config_from_user(netio::UdpReceiverConfig& out,
-                                    const roc_interface_config& in) {
+ROC_ATTR_NO_SANITIZE_UB bool interface_config_from_user(netio::UdpConfig& out,
+                                                        const roc_interface_config& in) {
     if (in.outgoing_address[0] != '\0') {
         if (!out.bind_address.set_host_port_auto(in.outgoing_address, 0)) {
             roc_log(LogError,
@@ -286,13 +265,15 @@ receiver_interface_config_from_user(netio::UdpReceiverConfig& out,
         strcpy(out.multicast_interface, in.multicast_group);
     }
 
-    out.reuseaddr = (in.reuse_address != 0);
+    out.enable_reuseaddr = (in.reuse_address != 0);
 
     return true;
 }
 
 ROC_ATTR_NO_SANITIZE_UB
-bool sample_spec_from_user(audio::SampleSpec& out, const roc_media_encoding& in) {
+bool sample_spec_from_user(audio::SampleSpec& out,
+                           const roc_media_encoding& in,
+                           bool is_network) {
     if (in.rate != 0) {
         out.set_sample_rate(in.rate);
     } else {
@@ -302,7 +283,7 @@ bool sample_spec_from_user(audio::SampleSpec& out, const roc_media_encoding& in)
         return false;
     }
 
-    if (in.format != ROC_FORMAT_PCM_FLOAT32) {
+    if (!sample_format_from_user(out, in.format, is_network)) {
         roc_log(LogError,
                 "bad configuration: invalid roc_media_encoding.format:"
                 " should be valid enum value");
@@ -350,30 +331,43 @@ bool sample_spec_from_user(audio::SampleSpec& out, const roc_media_encoding& in)
     return true;
 }
 
+ROC_ATTR_NO_SANITIZE_UB
+bool sample_format_from_user(audio::SampleSpec& out, roc_format in, bool is_network) {
+    switch (enum_from_user(in)) {
+    case ROC_FORMAT_PCM_FLOAT32:
+        out.set_sample_format(audio::SampleFormat_Pcm);
+        // TODO(gh-608): use PcmFormat_Float32_Be instead of PcmFormat_SInt16_Be
+        out.set_pcm_format(is_network ? audio::PcmFormat_SInt16_Be
+                                      : audio::PcmFormat_Float32);
+        return true;
+    }
+
+    return false;
+}
+
 ROC_ATTR_NO_SANITIZE_UB
 bool channel_set_from_user(audio::ChannelSet& out,
                            roc_channel_layout in,
                            unsigned int in_tracks) {
+    out.clear();
+
     switch (enum_from_user(in)) {
     case ROC_CHANNEL_LAYOUT_MULTITRACK:
-        out.clear();
         out.set_layout(audio::ChanLayout_Multitrack);
         out.set_order(audio::ChanOrder_None);
-        out.set_channel_range(0, in_tracks - 1, true);
+        out.set_range(0, in_tracks - 1);
         return true;
 
     case ROC_CHANNEL_LAYOUT_MONO:
-        out.clear();
         out.set_layout(audio::ChanLayout_Surround);
         out.set_order(audio::ChanOrder_Smpte);
-        out.set_channel_mask(audio::ChanMask_Surround_Mono);
+        out.set_mask(audio::ChanMask_Surround_Mono);
         return true;
 
     case ROC_CHANNEL_LAYOUT_STEREO:
-        out.clear();
         out.set_layout(audio::ChanLayout_Surround);
         out.set_order(audio::ChanOrder_Smpte);
-        out.set_channel_mask(audio::ChanMask_Surround_Stereo);
+        out.set_mask(audio::ChanMask_Surround_Stereo);
         return true;
     }
 
@@ -383,6 +377,7 @@ bool channel_set_from_user(audio::ChannelSet& out,
 ROC_ATTR_NO_SANITIZE_UB
 bool clock_source_from_user(bool& out_timing, roc_clock_source in) {
     switch (enum_from_user(in)) {
+    case ROC_CLOCK_SOURCE_DEFAULT:
     case ROC_CLOCK_SOURCE_EXTERNAL:
         out_timing = false;
         return true;
@@ -396,15 +391,15 @@ bool clock_source_from_user(bool& out_timing, roc_clock_source in) {
 }
 
 ROC_ATTR_NO_SANITIZE_UB
-bool clock_sync_backend_from_user(bool& out_fe, roc_clock_sync_backend in) {
+bool latency_tuner_backend_from_user(audio::LatencyTunerBackend& out,
+                                     roc_latency_tuner_backend in) {
     switch (enum_from_user(in)) {
-    case ROC_CLOCK_SYNC_BACKEND_DISABLE:
-        out_fe = false;
+    case ROC_LATENCY_TUNER_BACKEND_DEFAULT:
+        out = audio::LatencyTunerBackend_Default;
         return true;
 
-    case ROC_CLOCK_SYNC_BACKEND_DEFAULT:
-    case ROC_CLOCK_SYNC_BACKEND_NIQ:
-        out_fe = true;
+    case ROC_LATENCY_TUNER_BACKEND_NIQ:
+        out = audio::LatencyTunerBackend_Niq;
         return true;
     }
 
@@ -412,16 +407,23 @@ bool clock_sync_backend_from_user(bool& out_fe, roc_clock_sync_backend in) {
 }
 
 ROC_ATTR_NO_SANITIZE_UB
-bool clock_sync_profile_from_user(audio::FreqEstimatorProfile& out,
-                                  roc_clock_sync_profile in) {
+bool latency_tuner_profile_from_user(audio::LatencyTunerProfile& out,
+                                     roc_latency_tuner_profile in) {
     switch (enum_from_user(in)) {
-    case ROC_CLOCK_SYNC_PROFILE_DEFAULT:
-    case ROC_CLOCK_SYNC_PROFILE_RESPONSIVE:
-        out = audio::FreqEstimatorProfile_Responsive;
+    case ROC_LATENCY_TUNER_PROFILE_DEFAULT:
+        out = audio::LatencyTunerProfile_Default;
         return true;
 
-    case ROC_CLOCK_SYNC_PROFILE_GRADUAL:
-        out = audio::FreqEstimatorProfile_Gradual;
+    case ROC_LATENCY_TUNER_PROFILE_INTACT:
+        out = audio::LatencyTunerProfile_Intact;
+        return true;
+
+    case ROC_LATENCY_TUNER_PROFILE_RESPONSIVE:
+        out = audio::LatencyTunerProfile_Responsive;
+        return true;
+
+    case ROC_LATENCY_TUNER_PROFILE_GRADUAL:
+        out = audio::LatencyTunerProfile_Gradual;
         return true;
     }
 
@@ -603,37 +605,51 @@ bool proto_to_user(roc_protocol& out, address::Protocol in) {
 }
 
 ROC_ATTR_NO_SANITIZE_UB
-void receiver_slot_metrics_to_user(roc_receiver_metrics& out,
-                                   const pipeline::ReceiverSlotMetrics& in) {
-    out.num_sessions = (unsigned)in.num_sessions;
-}
-
-ROC_ATTR_NO_SANITIZE_UB
-void receiver_session_metrics_to_user(
-    const pipeline::ReceiverSessionMetrics& sess_metrics,
-    size_t sess_index,
-    void* sess_arg) {
-    roc_session_metrics& out = *((roc_session_metrics*)sess_arg + sess_index);
+void receiver_slot_metrics_to_user(const pipeline::ReceiverSlotMetrics& slot_metrics,
+                                   void* slot_arg) {
+    roc_receiver_metrics& out = *(roc_receiver_metrics*)slot_arg;
 
     memset(&out, 0, sizeof(out));
 
-    if (sess_metrics.latency.niq_latency > 0) {
-        out.niq_latency = (unsigned long long)sess_metrics.latency.niq_latency;
-    }
+    out.connection_count = (unsigned)slot_metrics.num_participants;
+}
 
-    if (sess_metrics.latency.e2e_latency > 0) {
-        out.e2e_latency = (unsigned long long)sess_metrics.latency.e2e_latency;
+ROC_ATTR_NO_SANITIZE_UB
+void receiver_participant_metrics_to_user(
+    const pipeline::ReceiverParticipantMetrics& party_metrics,
+    size_t party_index,
+    void* party_arg) {
+    roc_connection_metrics& out = *((roc_connection_metrics*)party_arg + party_index);
+
+    memset(&out, 0, sizeof(out));
+
+    if (party_metrics.latency.e2e_latency > 0) {
+        out.e2e_latency = (unsigned long long)party_metrics.latency.e2e_latency;
     }
 }
 
 ROC_ATTR_NO_SANITIZE_UB
-void sender_metrics_to_user(roc_sender_metrics& out,
-                            const pipeline::SenderSlotMetrics& in_slot,
-                            const pipeline::SenderSessionMetrics& in_sess) {
+void sender_slot_metrics_to_user(const pipeline::SenderSlotMetrics& slot_metrics,
+                                 void* slot_arg) {
+    roc_sender_metrics& out = *(roc_sender_metrics*)slot_arg;
+
     memset(&out, 0, sizeof(out));
 
-    (void)in_slot;
-    (void)in_sess;
+    out.connection_count = (unsigned)slot_metrics.num_participants;
+}
+
+ROC_ATTR_NO_SANITIZE_UB
+void sender_participant_metrics_to_user(
+    const pipeline::SenderParticipantMetrics& party_metrics,
+    size_t party_index,
+    void* party_arg) {
+    roc_connection_metrics& out = *((roc_connection_metrics*)party_arg + party_index);
+
+    memset(&out, 0, sizeof(out));
+
+    if (party_metrics.latency.e2e_latency > 0) {
+        out.e2e_latency = (unsigned long long)party_metrics.latency.e2e_latency;
+    }
 }
 
 ROC_ATTR_NO_SANITIZE_UB
@@ -648,6 +664,9 @@ LogLevel log_level_from_user(roc_log_level in) {
     case ROC_LOG_INFO:
         return LogInfo;
 
+    case ROC_LOG_NOTE:
+        return LogNote;
+
     case ROC_LOG_DEBUG:
         return LogDebug;
 
@@ -669,6 +688,9 @@ roc_log_level log_level_to_user(LogLevel in) {
     case LogInfo:
         return ROC_LOG_INFO;
 
+    case LogNote:
+        return ROC_LOG_NOTE;
+
     case LogDebug:
         return ROC_LOG_DEBUG;
 
