@@ -33,6 +33,7 @@ TEST_GROUP(sender) {
         sender_config.frame_encoding.rate = 44100;
         sender_config.frame_encoding.format = ROC_FORMAT_PCM_FLOAT32;
         sender_config.frame_encoding.channels = ROC_CHANNEL_LAYOUT_STEREO;
+        sender_config.packet_encoding = ROC_PACKET_ENCODING_AVP_L16_STEREO;
         sender_config.fec_encoding = ROC_FEC_ENCODING_DISABLE;
     }
 
@@ -388,14 +389,32 @@ TEST(sender, bad_args) {
                                  source_endpoint)
               == 0);
 
-        roc_sender_metrics metrics;
-        memset(&metrics, 0, sizeof(metrics));
+        roc_sender_metrics send_metrics;
+        roc_connection_metrics conn_metrics;
+        size_t conn_metrics_count = 1;
 
-        CHECK(roc_sender_query(NULL, ROC_SLOT_DEFAULT, &metrics) == -1);
-        CHECK(roc_sender_query(sender, 999, &metrics) == -1);
-        CHECK(roc_sender_query(sender, ROC_SLOT_DEFAULT, NULL) == -1);
+        // bad
+        CHECK(roc_sender_query(NULL, ROC_SLOT_DEFAULT, &send_metrics, &conn_metrics,
+                               &conn_metrics_count)
+              == -1);
+        CHECK(
+            roc_sender_query(NULL, 999, &send_metrics, &conn_metrics, &conn_metrics_count)
+            == -1);
+        CHECK(
+            roc_sender_query(sender, ROC_SLOT_DEFAULT, &send_metrics, &conn_metrics, NULL)
+            == -1);
 
-        CHECK(roc_sender_query(sender, ROC_SLOT_DEFAULT, &metrics) == 0);
+        // good
+        CHECK(roc_sender_query(sender, ROC_SLOT_DEFAULT, &send_metrics, NULL, NULL) == 0);
+        CHECK(roc_sender_query(sender, ROC_SLOT_DEFAULT, NULL, &conn_metrics,
+                               &conn_metrics_count)
+              == 0);
+        CHECK(roc_sender_query(sender, ROC_SLOT_DEFAULT, &send_metrics, NULL,
+                               &conn_metrics_count)
+              == 0);
+        CHECK(roc_sender_query(sender, ROC_SLOT_DEFAULT, &send_metrics, &conn_metrics,
+                               &conn_metrics_count)
+              == 0);
 
         CHECK(roc_endpoint_deallocate(source_endpoint) == 0);
         LONGS_EQUAL(0, roc_sender_close(sender));
